################################################################################
# Model cooling of coffee in a cup with a simple lumped model
#
#    mc cc dTc/dt = h A (Tout - Tc)
#
# where: mc = mass of coffee in cup
#        cc = heat capacity of coffee
#        Tc = temperature of coffee
#      Tout = temperature of environment
#         h = heat transfer coefficient
#         A = surface area of coffee cup (including upper surface)
#
#  Note that dividing through by mc cc gives the equation implemented:
#
#         dTc/dt = h A / (mc cc) (Tout - Tc)
#

import matplotlib.pyplot as plt
import math
import IVP_coffee    

def solve_coffee(coffee_IVP, dt):
    # Sets initial condition
    t0 = coffee_IVP.get_tI()
    v0 = coffee_IVP.get_uI()[0] # the [0] is because get_uI returns a float list

    t = [t0]
    v = [v0]
    
    # Loop from from t=t0 to t>=tF
    tn = t[0]
    vn = v[0]

    while (tn<tF):
        
        
        # # Update solution and time (Forward Euler)
        # fn = coffee_IVP.evalf([vn], tn)[0]
        # vn1 = vn + dt*fn
        # tn1 = tn + dt
        
        # Update solution and time (RK2)
        fn = coffee_IVP.evalf([vn], tn)[0]
        va = vn + dt*fn/2.

        fa = coffee_IVP.evalf([va], tn+dt/2.)[0]
        vn1 = vn + dt*fa
        tn1 = tn + dt       
        
        # Append to v and t
        v.append(vn1)
        t.append(tn1) # store time in minutes
        
        # Set vn and tn for next iteration
        vn = vn1
        tn = tn1
    
    return t, v


p = {}
p['mc']   = 0.35 # kg
p['cc']   = 4200.0 # J / (kg C)
p['h']    = 5.0 # W/(m^2 C)
p['A']    = 0.04 # m^2
p['Tout'] = 25.0 # C
        
TcI   = 85.0 # Initial temperature of coffee (C)
tFmin = 700.0 # final time to simulate to (min)
dtmin = 5e-1 # time increment to give solutions at (min)

# Convert times to seconds
tF = tFmin*60
dt = dtmin*60

# Initialize CoffeeIVP object
coffee_IVP = IVP_coffee.CoffeeIVP([TcI], 0.0, tF, p)

# Solve coffee IVP
t, Tc = solve_coffee(coffee_IVP, dt)

# Calculate exact solution and error
u = []
e = []

h    = p['h']
A    = p['A']
mc   = p['mc']
cc   = p['cc']
lam  = -h*A/(mc*cc)
Tout = p['Tout']

for n in range(len(t)):
    ts = t[n]
    t[n] = t[n]/60.0 # convert to minutes
    un = Tout + (TcI-Tout)*math.exp(lam*ts) # this is the exact solution
    en = abs(Tc[n]-un)
    u.append(un)
    e.append(en)

# Plot   
plt.subplot(2,1,1)
plt.scatter(t,Tc,marker='o',label='numerical')
plt.ylabel('$T_c$ (C)')
plt.grid(True)
    
plt.scatter(t,u,marker='x',label='exact')
plt.legend()

plt.subplot(2,1,2)
plt.scatter(t,e,marker='o')
plt.xlabel('t (min)')
plt.ylabel('error (C)')
plt.grid(True)


        
# RK2
        # # Update solution and time
        # fn = coffee_IVP.evalf([vn], tn)[0]
        # aa = dt*fn
        # va = vn + aa/2.
        # fa = coffee_IVP.evalf([va], tn+dt/2.)[0]
        # bb = dt*fa
        # vn1 = vn + bb
        # tn1 = tn + dt